@@ -236,17 +236,20 @@ PrimeConnection::close_connection (void)
             ssize_t rv = write (m_in_fd,
                                 command + (len - remaining),
                                 remaining);
-            switch (errno) {
-            case EBADF:
-            case EINVAL:
-            case EPIPE:
-                remaining = 0;
-                if (m_err_msg.empty ())
-                    set_error_message (COMMUNICATION_FAILD, errno);
-                break;
-            default:
+            if (rv < 0) {
+                switch (errno) {
+                case EBADF:
+                case EINVAL:
+                case EPIPE:
+                    remaining = 0;
+                    if (m_err_msg.empty ())
+                        set_error_message (COMMUNICATION_FAILD, errno);
+                    break;
+                default:
+                    break;
+                }
+            } else {
                 remaining -= rv;
-                break;
             }
         }
 #endif
@@ -542,7 +545,6 @@ PrimeConnection::send_command (const char *command,
         goto ERROR;
     }
 #else
-    ssize_t rv;
     size_t len, remaining;
     len = remaining = str.length ();
 
@@ -550,18 +552,21 @@ PrimeConnection::send_command (const char *command,
         ssize_t rv = write (m_in_fd,
                             str.c_str() + (len - remaining),
                             remaining);
-        switch (errno) {
-        case EBADF:
-        case EINVAL:
-        case EPIPE:
-            clean_child ();
-            if (m_err_msg.empty ())
-                set_error_message (COMMUNICATION_FAILD, errno);
-            goto ERROR;
-            break;
-        default:
+        if (rv < 0) {
+            switch (errno) {
+            case EBADF:
+            case EINVAL:
+            case EPIPE:
+                clean_child ();
+                if (m_err_msg.empty ())
+                    set_error_message (COMMUNICATION_FAILD, errno);
+                goto ERROR;
+                break;
+            default:
+                break;
+            }
+        } else {
             remaining -= rv;
-            break;
         }
     }
 #endif
@@ -574,8 +579,8 @@ PrimeConnection::send_command (const char *command,
 
     char buf[4096];
     while (true) {
-        rv = read (m_out_fd, buf, 4095);
-        if (rv <= 0) {
+        ssize_t rv = read (m_out_fd, buf, 4095);
+        if (rv < 0) {
             switch (errno) {
             case EBADF:
             case EINVAL:
@@ -660,15 +665,19 @@ PrimeConnection::write_all (int fd, const char *buf, int size)
         return false;
 
     while (remaining > 0) {
-        int rv = write (fd, buf + (size - remaining), remaining);
-        switch (errno) {
-        case EBADF:
-        case EINVAL:
-        case EPIPE:
-            return false;
-        default:
+        ssize_t rv = write (fd, buf + (size - remaining), remaining);
+
+        if (rv < 0) {
+            switch (errno) {
+            case EBADF:
+            case EINVAL:
+            case EPIPE:
+                return false;
+            default:
+                break;
+            }
+        } else {
             remaining -= rv;
-            break;
         }
     }
 
